@@ -5,7 +5,7 @@
 
 #include "links.h"
 
-#ifdef DONT_USE_AF_UNIX
+#ifndef USE_AF_UNIX
 
 int s_unix_fd = -1;
 
@@ -20,7 +20,22 @@ void af_unix_close(void)
 
 #else
 
-#ifdef USE_AF_UNIX
+#ifdef OS2
+#ifndef HAVE_SYS_UN_H
+#define HAVE_SYS_UN_H
+#endif
+struct os2_sockaddr_un {
+	u_short sun_family;
+	char sun_path[108];
+};
+#define sockaddr_un	os2_sockaddr_un
+#ifndef AF_UNIX
+#define AF_UNIX	1
+#endif
+#ifndef PF_UNIX
+#define PF_UNIX	1
+#endif
+#elif defined(HAVE_SYS_UN_H)
 #include <sys/un.h>
 #endif
 
@@ -34,7 +49,7 @@ static void af_unix_connection(void *);
 
 union address {
 	struct sockaddr s;
-#ifdef USE_AF_UNIX
+#ifdef HAVE_SYS_UN_H
 	struct sockaddr_un suni;
 #endif
 	struct sockaddr_in sin;
@@ -66,49 +81,16 @@ static struct links_handshake {
 	if ((r = hard_read(hndl, (unsigned char *)&received_handshake, sz)) != (sz) || memcmp(&received_handshake, &links_handshake, sz))
 
 
-#ifdef USE_AF_UNIX
-
-static int get_address(unsigned char *name)
+#ifdef OS2
+static inline int have_pf_unix(void)
 {
-	unsigned char *path;
-	if (!links_home) return -1;
-	path = stracpy(links_home);
-	add_to_strn(&path, cast_uchar LINKS_SOCK_NAME);
-	if (anonymous) {
-		add_to_strn(&path, cast_uchar LINKS_ANONYMOUS_SOCK_SUFFIX);
-	}
-	if (name) {
-		unsigned char *n = stracpy(name);
-		check_filename(&n);
-		add_to_strn(&path, cast_uchar "-");
-		add_to_strn(&path, n);
-		mem_free(n);
-	}
-	s_unix_l = (socklen_t)((unsigned char *)&s_unix.suni.sun_path - (unsigned char *)&s_unix.suni + strlen(cast_const_char path) + 1);
-	if (strlen(cast_const_char path) > sizeof(union address) || (size_t)s_unix_l > sizeof(union address)) {
-		mem_free(path);
-		return -1;
-	}
-	memset(&s_unix, 0, sizeof s_unix);
-	s_unix.suni.sun_family = AF_UNIX;
-	strcpy(cast_char s_unix.suni.sun_path, cast_const_char path);
-	mem_free(path);
-	return PF_UNIX;
+	s_unix_fd = c_socket(PF_UNIX, SOCK_STREAM, 0);
+	return s_unix_fd >= 0;
 }
+#endif
 
-static void unlink_unix(void)
-{
-	int rs;
-	/*debug("unlink: %s", s_unix.suni.sun_path);*/
-	EINTRLOOP(rs, unlink(s_unix.suni.sun_path));
-	if (rs) {
-		/*perror("unlink");*/
-	}
-}
-
-#else
-
-static int get_address(unsigned char *name)
+#if !defined(HAVE_SYS_UN_H) || defined(OS2)
+static inline int get_address_localhost(unsigned char *name)
 {
 	unsigned short port;
 	if (!name) {
@@ -136,12 +118,64 @@ static int get_address(unsigned char *name)
 	/*debug("get address %d", port);*/
 	return PF_INET;
 }
+#endif
+
+static int get_address(unsigned char *name)
+{
+#ifdef HAVE_SYS_UN_H
+	unsigned char *path;
+#ifdef OS2
+	if (!have_pf_unix()) {
+		return get_address_localhost(name);
+	}
+	path = stracpy(cast_uchar "\\socket\\links\\");
+#else
+	if (!links_home) return -1;
+	path = stracpy(links_home);
+#endif
+	add_to_strn(&path, cast_uchar LINKS_SOCK_NAME);
+	if (anonymous) {
+		add_to_strn(&path, cast_uchar LINKS_ANONYMOUS_SOCK_SUFFIX);
+	}
+	if (name) {
+		unsigned char *n = stracpy(name);
+		check_filename(&n);
+		add_to_strn(&path, cast_uchar "-");
+		add_to_strn(&path, n);
+		mem_free(n);
+	}
+#ifdef OS2
+	s_unix_l = sizeof(struct sockaddr_un);
+#else
+	s_unix_l = (socklen_t)((unsigned char *)&s_unix.suni.sun_path - (unsigned char *)&s_unix.suni + strlen(cast_const_char path) + 1);
+#endif
+	if (strlen(cast_const_char path) > sizeof(union address) || (size_t)s_unix_l > sizeof(union address)) {
+		mem_free(path);
+		return -1;
+	}
+	memset(&s_unix, 0, sizeof s_unix);
+	s_unix.suni.sun_family = AF_UNIX;
+	strcpy(cast_char s_unix.suni.sun_path, cast_const_char path);
+	mem_free(path);
+	return PF_UNIX;
+#else
+	return get_address_localhost(name);
+#endif
+}
 
 static void unlink_unix(void)
 {
-}
-
+#if defined(HAVE_SYS_UN_H) && !defined(OS2)
+	if (s_unix.s.sa_family == AF_UNIX) {
+		int rs;
+		/*debug("unlink: %s", s_unix.suni.sun_path);*/
+		EINTRLOOP(rs, unlink(s_unix.suni.sun_path));
+		if (rs) {
+			/*perror("unlink");*/
+		}
+	}
 #endif
+}
 
 int bind_to_af_unix(unsigned char *name)
 {
@@ -157,10 +191,15 @@ int bind_to_af_unix(unsigned char *name)
 	safe_strncpy(links_handshake.system_name, system_name, sizeof links_handshake.system_name);
 	links_handshake.system_id = SYSTEM_ID;
 	links_handshake.sizeof_long = sizeof(long);
-	if ((af = get_address(name)) == -1) return -1;
+	if ((af = get_address(name)) == -1) {
+		close_socket(&s_unix_fd);
+		return -1;
+	}
 	again:
-	s_unix_fd = c_socket(af, SOCK_STREAM, 0);
-	if (s_unix_fd == -1) return -1;
+	if (s_unix_fd == -1) {
+		s_unix_fd = c_socket(af, SOCK_STREAM, 0);
+		if (s_unix_fd == -1) return -1;
+	}
 #if defined(SOL_SOCKET) && defined(SO_REUSEADDR)
 	EINTRLOOP(rs, setsockopt(s_unix_fd, SOL_SOCKET, SO_REUSEADDR, (void *)&a1, sizeof a1));
 #endif
