@@ -3,7 +3,7 @@
  * This file is a part of the Links program, released under GPL
  */
 
-#if defined(__BEOS__) || defined(__HAIKU__)
+#if defined(__BEOS__)
 
 #include "com-defs.h"
 
