@@ -48,3 +48,8 @@
 #define t8c unsigned long long
 #endif /* #if sizeof(short) */
 
+#ifdef t8c
+#if SIZEOF_UNSIGNED_LONG >= 8 || defined(__x86_64__) || defined(__aarch64__)
+#define t8c_is_efficient
+#endif
+#endif
