@@ -232,14 +232,12 @@ void detach_cache_entry(struct cache_entry *e)
 
 static void mem_free_fragment(struct fragment *f)
 {
-	size_t s = (size_t)f->length;
 	mem_free(f);
-	s += sizeof(struct fragment);
 }
 
 #define sf(x) e->data_size += (x), cache_size += (my_uintptr_t)(x)
 
-#define C_ALIGN(x) ((((x) + sizeof(struct fragment) + alloc_overhead) | (page_size - 1)) - sizeof(struct fragment) - alloc_overhead)
+#define C_ALIGN(x) ((((x) + (sizeof(struct fragment) + alloc_overhead - 1)) | (page_size - 1)) - (sizeof(struct fragment) + alloc_overhead - 1))
 
 int add_fragment(struct cache_entry *e, off_t offset, const unsigned char *data, off_t length)
 {
@@ -477,6 +475,11 @@ void delete_cache_entry(struct cache_entry *e)
 	mem_free(e);
 }
 
+void finish_cache_entry(struct cache_entry *e)
+{
+	e->count = cache_count++;
+}
+
 static int shrink_file_cache(int u)
 {
 	int r = 0;
