@@ -165,7 +165,7 @@ static void add_utf_8(struct conv_table *ct, int u, unsigned char *str)
 		else {
 			struct conv_table *nct;
 			if (ct[*p].u.str != no_str) {
-				internal("bad utf encoding #1");
+				internal_error("bad utf encoding #1");
 				return;
 			}
 			nct = mem_alloc(sizeof(struct conv_table) * 256);
@@ -178,7 +178,7 @@ static void add_utf_8(struct conv_table *ct, int u, unsigned char *str)
 		p++;
 	}
 	if (ct[*p].t) {
-		internal("bad utf encoding #2");
+		internal_error("bad utf encoding #2");
 		return;
 	}
 	if (ct[*p].u.str == no_str) ct[*p].u.str = str;
@@ -625,6 +625,6 @@ unsigned char *cp_strchr(int charset, unsigned char *str, unsigned chr)
 void init_charset(void)
 {
 	utf8_table = get_cp_index(cast_uchar "UTF-8");
-	if (utf8_table == -1) internal("no UTF-8 charset");
+	if (utf8_table == -1) internal_error("no UTF-8 charset");
 	bookmarks_codepage = utf8_table;
 }
