@@ -110,9 +110,7 @@ int socket_and_bind(int pf, unsigned char *address)
 			struct sockaddr_in sa;
 			unsigned char addr[4];
 			if (numeric_ip_address(address, addr) == -1) {
-				EINTRLOOP(rs, close(s));
-				errno = EINVAL;
-				return -1;
+				goto bind_to_iface;
 			}
 			memset(&sa, 0, sizeof sa);
 			sa.sin_family = AF_INET;
@@ -133,9 +131,7 @@ int socket_and_bind(int pf, unsigned char *address)
 			unsigned char addr[16];
 			unsigned scope;
 			if (numeric_ipv6_address(address, addr, &scope) == -1) {
-				EINTRLOOP(rs, close(s));
-				errno = EINVAL;
-				return -1;
+				goto bind_to_iface;
 			}
 			memset(&sa, 0, sizeof sa);
 			sa.sin6_family = AF_INET6;
@@ -162,6 +158,16 @@ int socket_and_bind(int pf, unsigned char *address)
 		}
 	}
 	return s;
+bind_to_iface:
+	errno = EINVAL;
+#ifdef SO_BINDTODEVICE
+	EINTRLOOP(rs, setsockopt(s, SOL_SOCKET, SO_BINDTODEVICE, address, (socklen_t)(strlen(cast_const_char address) + 1)));
+	if (!rs) {
+		return s;
+	}
+#endif
+	EINTRLOOP(rs, close(s));
+	return -1;
 }
 
 void close_socket(int *s)
@@ -187,7 +193,6 @@ struct conn_info {
 void make_connection(struct connection *c, int port, int *sock, void (*func)(struct connection *))
 {
 	int socks_port = -1;
-	int as;
 	unsigned char *host;
 	size_t sl;
 	struct conn_info *b;
@@ -239,14 +244,17 @@ void make_connection(struct connection *c, int port, int *sock, void (*func)(str
 		b->l.addr_index = c->last_lookup_state.addr_index;
 		b->l.dont_try_more_servers = 1;
 		dns_found(c, 0);
-		as = 0;
-	} else if (c->no_cache >= NC_RELOAD) {
-		as = find_host_no_cache(host, &b->l.addr, &c->dnsquery, dns_found, c);
 	} else {
-		as = find_host(host, &b->l.addr, &c->dnsquery, dns_found, c);
+		int q = -1;
+		if (c->no_cache < NC_RELOAD) {
+			q = find_host_in_cache(host, &b->l.addr, &c->dnsquery, dns_found, c);
+		}
+		if (q) {
+			setcstate(c, S_DNS);
+			find_host_no_cache(host, c->doh, &b->l.addr, &c->dnsquery, dns_found, c);
+		}
 	}
 	mem_free(host);
-	if (as) setcstate(c, S_DNS);
 }
 
 int is_ipv6(int h)
@@ -610,16 +618,13 @@ void retry_connect(struct connection *c, int err, int ssl_downgrade)
 		return;
 	}
 	b->l.addr_index++;
-#if MAX_ADDRESSES > 1
 	if (b->l.addr_index < b->l.addr.n && !b->l.dont_try_more_servers) {
 		if (b->l.addr_index == 1)
 			rotate_addresses(&b->l.addr);
 		log_string(cast_uchar "\nNEXT ADDRESS\n");
 		close_socket(b->sock);
 		try_connect(c);
-	} else
-#endif
-	{
+	} else {
 		dns_clear_host(b->host);
 		setcstate(c, b->first_error);
 		retry_connection(c);
@@ -702,11 +707,9 @@ static void try_connect(struct connection *c)
 		}
 		set_handlers(s, NULL, connected, c);
 		setcstate(c, !b->l.addr_index ? S_CONN : S_CONN_ANOTHER);
-#if MAX_ADDRESSES > 1
 		if (b->l.addr.n > 1 && (is_connection_restartable(c) || max_tries == 1)) {
 			set_connection_timeout(c);
 		}
-#endif
 	} else {
 		connected(c);
 	}
@@ -783,6 +786,7 @@ static void connected(void *c_)
 		}
 		c->ssl = getSSL();
 		if (!c->ssl) {
+			c->ssl = DUMMY;
 			ret1 = ret2 = 0;
 			mem_free(h);
 			goto ssl_error;
@@ -845,7 +849,6 @@ skip_numeric_address:
 
 static void update_dns_priority(struct connection *c)
 {
-#if MAX_ADDRESSES > 1
 	struct conn_info *b = c->newconn;
 	if (!b->l.dont_try_more_servers && b->host[0]) {
 		if (b->l.addr_index) {
@@ -856,7 +859,6 @@ static void update_dns_priority(struct connection *c)
 		}
 		b->l.dont_try_more_servers = 1;
 	}
-#endif
 }
 
 static void connected_callback(struct connection *c)
