@@ -160,8 +160,10 @@ static void add_row_gray(unsigned *my_restrict row_buf, unsigned char *my_restri
 ATTR_NOINLINE
 #endif
 static void add_col_color(scale_t *my_restrict col_buf,
-#if defined(__GNUC__) && __GNUC__ == 3
+#ifdef __GNUC__
+#if __GNUC__ == 3
 	volatile
+#endif
 #endif
 	unsigned short *my_restrict ptr,
 	size_t line_skip, size_t n, ulonglong weight)
@@ -194,8 +196,10 @@ static void add_col_color(scale_t *my_restrict col_buf,
 /* n is in pixels. pixel is 3 unsigned shorts in series */
  /* We assume unsigned short holds at least 16 bits. */
 static void add_row_color(scale_t *my_restrict row_buf,
-#if defined(__GNUC__) && __GNUC__ == 3
+#ifdef __GNUC__
+#if __GNUC__ == 3
 	volatile
+#endif
 #endif
 	unsigned short *my_restrict ptr,
 	size_t n, ulonglong weight)
@@ -1992,11 +1996,14 @@ bypass_freetype:
  * at least 1 apart
  * Otherwise g_print_text will print nonsense (but won't segfault)
  */
-static void get_underline_pos(int height, int *top, int *bottom)
+static void get_line_pos(int height, int *top, int *bottom, unsigned style)
 {
 	int thickness, baseline;
 	thickness=(height+15)/16;
-	baseline=height/7;
+	if (style == FF_STRIKE)
+		baseline=height/2 - height/12;
+	else
+		baseline=height/7;
 	if (baseline<=0) baseline=1;
 	if (thickness>baseline) thickness=baseline;
 	*top=height-baseline;
@@ -2006,38 +2013,39 @@ static void get_underline_pos(int height, int *top, int *bottom)
 /* *width will be advanced by the width of the text */
 void g_print_text(struct graphics_device *device, int x, int y, struct style *style, unsigned char *text, int *width)
 {
-	int top_underline, bottom_underline, original_width, my_width;
-	unsigned char original_flags;
+	int top_line, bottom_line, original_width, my_width;
+	unsigned char original_flags, line_style;
 	struct rect saved_clip;
 
 	if (y + style->height <= device->clip.y1 || y >= device->clip.y2)
 		goto o;
-	if (style->flags & FF_UNDERLINE) {
-		/* Underline */
+	if (style->flags & FF_UNDERLINE || style->flags & FF_STRIKE) {
+		/* Underline or strike */
 		if (!width) {
 		       width = &my_width;
 		       *width = 0;
 		}
 		original_flags = style->flags;
 		original_width = *width;
-		style->flags &= ~FF_UNDERLINE;
-		get_underline_pos(style->height, &top_underline, &bottom_underline);
-		restrict_clip_area(device, &saved_clip, 0, 0, device->size.x2, y + top_underline);
+		line_style = style->flags & FF_UNDERLINE ? FF_UNDERLINE : FF_STRIKE;
+		style->flags &= ~line_style;
+		get_line_pos(style->height, &top_line, &bottom_line, line_style);
+		restrict_clip_area(device, &saved_clip, 0, 0, device->size.x2, y + top_line);
 		g_print_text(device, x, y, style, text, width);
 		set_clip_area(device, &saved_clip);
-		if (bottom_underline - top_underline == 1) {
+		if (bottom_line - top_line == 1) {
 			/* Line */
-			drv->draw_hline(device, x, y + top_underline, safe_add(x, *width) - original_width, style->underline_color);
+			drv->draw_hline(device, x, y + top_line, safe_add(x, *width) - original_width, style->line_color);
 		} else {
 			/* Area */
-			drv->fill_area(device, x, y + top_underline, safe_add(x, *width) - original_width, y + bottom_underline, style->underline_color);
+			drv->fill_area(device, x, y + top_line, safe_add(x, *width) - original_width, y + bottom_line, style->line_color);
 		}
-		if (bottom_underline < style->height) {
-			/* Do the bottom half only if the underline is above
+		if (bottom_line < style->height) {
+			/* Do the bottom half only if the line is above
 			 * the bottom of the letters.
 			 */
 			*width = original_width;
-			restrict_clip_area(device, &saved_clip, 0, y + bottom_underline, device->size.x2, device->size.y2);
+			restrict_clip_area(device, &saved_clip, 0, y + bottom_line, device->size.x2, device->size.y2);
 			g_print_text(device, x, y, style, text, width);
 			set_clip_area(device, &saved_clip);
 		}
@@ -2271,9 +2279,11 @@ struct style *g_invert_style(struct style *old)
 #ifdef HAVE_FREETYPE
 	st->ft_face = old->ft_face;
 #endif
-	if (st->flags & FF_UNDERLINE) {
-		/* We have to get a foreground color for underlining */
-		st->underline_color = dip_get_color_sRGB((st->r1 << 16) | (st->g1 << 8) | (st->b1));
+	if (st->flags & FF_UNDERLINE || st->flags & FF_STRIKE) {
+		/* We have to get a foreground color for underlining and
+		 * striking
+		 */
+		st->line_color = dip_get_color_sRGB((st->r1 << 16) | (st->g1 << 8) | (st->b1));
 	}
 	st->mono_space = old->mono_space;
 	st->mono_height = old->mono_height;
@@ -2319,9 +2329,11 @@ struct style *g_get_style_font(int fg, int bg, int size, int fflags, unsigned ch
 	else
 		st->ft_face = freetype_get_font(font);
 #endif
-	if (fflags & FF_UNDERLINE) {
-		/* We have to get a foreground color for underlining */
-		st->underline_color = dip_get_color_sRGB(fg);
+	if (fflags & FF_UNDERLINE || fflags & FF_STRIKE) {
+		/* We have to get a foreground color for underlining and
+		 * striking
+		 */
+		st->line_color = dip_get_color_sRGB(fg);
 	}
 	if (fflags & FF_MONOSPACED)
 		load_metric(st, ' ', &st->mono_space, &st->mono_height);
