@@ -40,13 +40,10 @@ static void sigbreak(int sig)
 	break_pressed = 1;
 }
 
-static unsigned screen_x = 80;
-static unsigned screen_y = 25;
-
 void get_terminal_size(int *x, int *y)
 {
-	*x = screen_x = ScreenCols();
-	*y = screen_y = ScreenRows();
+	*x = ScreenCols();
+	*y = ScreenRows();
 }
 
 void handle_terminal_resize(void (*fn)(int, int), int *x, int *y)
@@ -241,7 +238,9 @@ x:
 
 void *handle_mouse(int cons, void (*fn)(void *, unsigned char *, int), void *data)
 {
-	dos_mouse_init(screen_backbuffer_x * 8, screen_backbuffer_y * 8);
+	int x, y;
+	get_terminal_size(&x, &y);
+	dos_mouse_init(x * 8, y * 8);
 	if (!dos_mouse_initialized) return NULL;
 	dos_mouse_show();
 	txt_mouse_handler = fn;
@@ -287,9 +286,10 @@ static int dos_mouse_event(void)
 	return 0;
 }
 
-void dos_save_screen(void)
+void save_terminal(void)
 {
 	unsigned char *sc;
+	want_draw();
 	screen_backbuffer_x = ScreenCols();
 	screen_backbuffer_y = ScreenRows();
 	screen_default_attr = ScreenAttrib;
@@ -302,10 +302,12 @@ void dos_save_screen(void)
 	ScreenRetrieve(sc);
 	ScreenGetCursor(&saved_cursor_y, &saved_cursor_x);
 	screen_backbuffer = sc;
+	done_draw();
 }
 
-void dos_restore_screen(void)
+void restore_terminal(void)
 {
+	want_draw();
 	if (screen_backbuffer) {
 		unsigned char *sc;
 		if (ScreenCols() == screen_backbuffer_x && ScreenRows() == screen_backbuffer_y) {
@@ -316,6 +318,7 @@ void dos_restore_screen(void)
 		screen_backbuffer = NULL;
 		mem_free(sc);
 	}
+	done_draw();
 }
 
 static void ansi_initialize(void)
@@ -323,8 +326,6 @@ static void ansi_initialize(void)
 	if (screen_initialized)
 		return;
 
-	dos_save_screen();
-
 	ScreenClear();
 	cursor_x = 0;
 	cursor_y = 0;
@@ -338,7 +339,6 @@ static void ansi_terminate(void)
 		return;
 
 	ScreenSetCursor(0, 0);
-	dos_restore_screen();
 	screen_initialized = 0;
 }
 
