@@ -31,6 +31,9 @@ extern struct graphics_driver pmshell_driver;
 #ifdef GRDRV_ATHEOS
 extern struct graphics_driver atheos_driver;
 #endif
+#ifdef GRDRV_HAIKU
+extern struct graphics_driver haiku_driver;
+#endif
 #ifdef GRDRV_GRX
 extern struct graphics_driver grx_driver;
 #endif
@@ -51,6 +54,9 @@ static struct graphics_driver *graphics_drivers[] = {
 #ifdef GRDRV_ATHEOS
 	&atheos_driver,
 #endif
+#ifdef GRDRV_HAIKU
+	&haiku_driver,
+#endif
 #ifndef SPAD
 #ifdef GRDRV_X
 	&x_driver,
@@ -480,4 +486,68 @@ void shutdown_virtual_devices(void)
 
 #endif
 
+#if defined(GRDRV_X) || defined(GRDRV_HAIKU)
+
+/* This is executed in a helper thread, so we must not use mem_alloc */
+
+static void addchr(unsigned char **str, size_t *l, unsigned char c)
+{
+	unsigned char *s;
+	if (!*str) return;
+	if ((*str)[*l]) *l = strlen(cast_const_char *str);
+	if (*l > MAXINT - 2) overalloc();
+	s = realloc(*str, *l + 2);
+	if (!s) {
+		free(*str);
+		*str = NULL;
+		return;
+	}
+	*str = s;
+	s[(*l)++] = c;
+	s[*l] = 0;
+}
+
+int x_exec(unsigned char *command, int fg)
+{
+	unsigned char *pattern, *final;
+	size_t i, j, l;
+	int retval;
+
+	if (!fg) {
+		retval = system(cast_const_char command);
+		return retval;
+	}
+
+	l = 0;
+	if (*drv->param->shell_term) {
+		pattern = cast_uchar strdup(cast_const_char drv->param->shell_term);
+	} else {
+		pattern = cast_uchar strdup(cast_const_char links_xterm());
+		if (*command) {
+			addchr(&pattern, &l, ' ');
+			addchr(&pattern, &l, '%');
+		}
+	}
+	if (!pattern) return -1;
+
+	final = cast_uchar strdup("");
+	l = 0;
+	for (i = 0; pattern[i]; i++) {
+		if (pattern[i] == '%') {
+			for (j = 0; j < strlen(cast_const_char command); j++)
+				addchr(&final, &l, command[j]);
+		} else {
+			addchr(&final, &l, pattern[i]);
+		}
+	}
+	free(pattern);
+	if (!final) return -1;
+
+	retval = system(cast_const_char final);
+	free(final);
+	return retval;
+}
+
+#endif
+
 #endif
