@@ -783,6 +783,146 @@ unsigned char *debug_stracpy(unsigned char *f, int l, const unsigned char *src)
 
 #endif
 
+#if defined(LEAK_DEBUG) && defined(LEAK_DEBUG_LIST)
+
+struct memory_entry {
+	struct alloc_header *ah;
+	my_uintptr_t cumulative_size;
+	my_uintptr_t n_blocks;
+};
+
+LIBC_CALLBACK static int memory_sort_file_line(const void *l1_, const void *l2_)
+{
+	const struct memory_entry *l1 = (const struct memory_entry *)l1_;
+	const struct memory_entry *l2 = (const struct memory_entry *)l2_;
+	int c;
+	c = strcmp(cast_const_char l1->ah->file, cast_const_char l2->ah->file);
+	if (c)
+		return c;
+	if (l1->ah->line != l2->ah->line)
+		return l1->ah->line - l2->ah->line;
+	return 0;
+}
+
+LIBC_CALLBACK static int memory_sort_cumulative_size(const void *l1_, const void *l2_)
+{
+	const struct memory_entry *l1 = (const struct memory_entry *)l1_;
+	const struct memory_entry *l2 = (const struct memory_entry *)l2_;
+	int c;
+	if (l1->cumulative_size < l2->cumulative_size)
+		return 1;
+	if (l1->cumulative_size > l2->cumulative_size)
+		return -1;
+	if (l1->n_blocks < l2->n_blocks)
+		return 1;
+	if (l1->n_blocks > l2->n_blocks)
+		return -1;
+	c = strcmp(cast_const_char l1->ah->file, cast_const_char l2->ah->file);
+	if (c)
+		return c;
+	if (l1->ah->line != l2->ah->line)
+		return l1->ah->line - l2->ah->line;
+	return 0;
+}
+
+static int num_digits(my_uintptr_t num)
+{
+	int n = 0;
+	do {
+		n++;
+		num /= 10;
+	} while (num);
+	return n;
+}
+
+unsigned char *get_top_memory(int mode, unsigned n)
+{
+	struct memory_entry *entries;
+	struct alloc_header *ah;
+	struct list_head *lah;
+	my_uintptr_t offs, offs2, mr, ln;
+	unsigned char *r;
+	int rl;
+	int nd = 0;	/* against warning */
+
+	entries = malloc(sizeof(struct memory_entry) * mem_blocks);
+	if (!entries)
+		return stracpy(cast_uchar "Not enough memory");
+
+	offs = 0;
+	foreach(struct alloc_header, ah, lah, memory_list) {
+		entries[offs].ah = ah;
+		entries[offs].cumulative_size = ah->size;
+		entries[offs].n_blocks = 1;
+		offs++;
+	}
+	if (offs) qsort(entries, offs, sizeof(struct memory_entry), (int (*)(const void *, const void *))memory_sort_file_line);
+	if (mode == GTM_MOST_ALLOCATED) {
+		offs2 = 0;
+		for (mr = 0; mr < offs; mr++) {
+			entries[offs2] = entries[mr];
+			while (mr + 1 < offs &&
+			      !strcmp(cast_const_char entries[mr].ah->file, cast_const_char entries[mr + 1].ah->file) &&
+			      entries[mr].ah->line == entries[mr + 1].ah->line) {
+				mr++;
+				entries[offs2].cumulative_size += entries[mr].ah->size;
+				entries[offs2].n_blocks++;
+			}
+			offs2++;
+		}
+	} else {
+		offs2 = offs;
+	}
+	if (offs2) qsort(entries, offs2, sizeof(struct memory_entry), (int (*)(const void *, const void *))memory_sort_cumulative_size);
+	r = init_str();
+	rl = 0;
+	if (offs2) {
+		nd = num_digits(entries[0].cumulative_size);
+	}
+	for (mr = 0, ln = 0; mr < offs2 && ln < n; mr++, ln++) {
+		int l1 = rl;
+		int d1;
+		unsigned char *g, *f = entries[mr].ah->file;
+		while ((g = cast_uchar strchr(cast_const_char f, '/'))) f = g + 1;
+		add_to_str(&r, &rl, f);
+		add_chr_to_str(&r, &rl, ':');
+		add_num_to_str(&r, &rl, entries[mr].ah->line);
+		add_chr_to_str(&r, &rl, ' ');
+		d1 = num_digits(entries[mr].cumulative_size);
+		while (rl < l1 + 24 + (nd - d1))
+			add_chr_to_str(&r, &rl, ' ');
+		add_num_to_str(&r, &rl, entries[mr].cumulative_size);
+		if (mode == GTM_MOST_ALLOCATED) {
+			add_to_str(&r, &rl, cast_uchar " / ");
+			add_num_to_str(&r, &rl, entries[mr].n_blocks);
+		} else if (mode == GTM_LARGEST_BLOCKS) {
+			my_uintptr_t mq;
+			for (mq = mr + 1; mq < offs2; mq++) {
+				if (strcmp(cast_const_char entries[mr].ah->file, cast_const_char entries[mq].ah->file))
+					break;
+				if (entries[mr].ah->line != entries[mq].ah->line)
+					break;
+				if (entries[mr].cumulative_size != entries[mq].cumulative_size)
+					break;
+			}
+			if (mq > mr + 1) {
+				add_to_str(&r, &rl, cast_uchar " x ");
+				add_num_to_str(&r, &rl, mq - mr);
+			}
+			mr = mq - 1;
+		}
+		add_chr_to_str(&r, &rl, '\n');
+	}
+	if (rl) {
+		rl--;
+		r[rl] = 0;
+	}
+	free(entries);
+	return r;
+}
+
+#endif
+
 #ifdef OOPS
 
 struct prot {
