@@ -205,7 +205,8 @@ again:
 	add_chr_to_str(p, l, ' ');
 }
 
-static unsigned char *get_filename(unsigned char *url)
+/* ATTR_NOINLINE is a workaround for a bogus warning on gcc 12 */
+ATTR_NOINLINE static unsigned char *get_filename(unsigned char *url)
 {
 	unsigned char *p, *m;
 	int ml;
