@@ -272,15 +272,15 @@ static struct ftp_connection_info *add_file_cmd_to_str(struct connection *c, int
 				sprintf(cast_char port_string, "%d,%d,%d,%d,%d,%d", pc[0], pc[1], pc[2], pc[3], pc[4], pc[5]);
 		}
 		if (strlen(cast_const_char port_string) >= sizeof(port_string))
-			internal("buffer overflow in get_pasv_socket_ipv6: %d > %d", (int)strlen(cast_const_char port_string), (int)sizeof(port_string));
-	}
+			internal_error("buffer overflow in get_pasv_socket_ipv6: %d > %d", (int)strlen(cast_const_char port_string), (int)sizeof(port_string));
 #ifdef HAVE_IPTOS
-	if (ftp_options.set_tos) {
-		int rx;
-		int on = IPTOS_THROUGHPUT;
-		EINTRLOOP(rx, setsockopt(c->sock2, IPPROTO_IP, IP_TOS, (char *)&on, sizeof(int)));
-	}
+		if (ftp_options.set_tos) {
+			int rx;
+			int on = IPTOS_THROUGHPUT;
+			EINTRLOOP(rx, setsockopt(c->sock2, IPPROTO_IP, IP_TOS, (char *)&on, sizeof(int)));
+		}
 #endif
+	}
 	dd = d;
 	while (*dd == '/') dd++;
 	de = cast_uchar strchr(cast_const_char dd, 0);
@@ -342,7 +342,7 @@ static void ftp_send_retr_req(struct connection *c, int state, int we_are_in_roo
 	struct ftp_connection_info *fi;
 	unsigned char *login;
 	int logl = 0;
-	set_connection_timeout(c);
+	set_connection_timeout_keepal(c);
 	login = init_str();
 	if (!c->info && !(fi = add_file_cmd_to_str(c, we_are_in_root))) {
 		mem_free(login);
@@ -462,7 +462,7 @@ static void ftp_retr_file(struct connection *c, struct read_buffer *rb)
 				}
 				goto rep;
 		}
-		internal("WHAT???");
+		internal_error("WHAT???");
 	}
 	g = get_ftp_response(c, rb, 2);
 	if (!g) { read_from_socket(c, c->sock1, rb, ftp_retr_file); setcstate(c, S_GETH); return; }
@@ -607,7 +607,7 @@ static int ftp_process_dirlist(struct cache_entry *ce, off_t *pos, int *d, unsig
 			if ((ee - sp != 1 || buf[sp] != '.') &&
 			    (ee - sp != 2 || buf[sp] != '.' || buf[sp + 1] != '.')) {
 				int i;
-				for (i = 0; i < sp; i++) add_chr_to_str(&str, &sl, ' ');
+				for (i = 0; i < sp; i++) add_to_str(&str, &sl, cast_uchar "&nbsp;");
 				add_to_str(&str, &sl, cast_uchar "<a href=\"../\">..</a>\n");
 			}
 		}
