@@ -13,6 +13,7 @@ struct f_data *init_formatted(struct document_options *opt)
 	scr->data = DUMMY;
 	scr->nlinks = 0;
 	scr->links = DUMMY;
+	if (!F) init_list(scr->compressed_chars);
 #ifdef JS
 	scr->nlink_events = 0;
 	scr->link_events = DUMMY;
@@ -21,7 +22,7 @@ struct f_data *init_formatted(struct document_options *opt)
 	init_list(scr->tags);
 	init_list(scr->nodes);
 #ifdef G
-	scr->n_images=0;
+	scr->n_images = 0;
 	init_list(scr->images);
 	scr->search_positions = DUMMY;
 	scr->search_lengths = DUMMY;
@@ -141,8 +142,14 @@ static void clear_formatted(struct f_data *scr)
 	}
 	mem_free(scr->link_events);
 #endif
-	if (!F) for (y = 0; y < scr->y; y++) mem_free(scr->data[y].d);
-	mem_free(scr->data);
+	if (!F) {
+		for (y = 0; y < scr->y; y++)
+			if (scr->data[y].allocated >= 0) {
+				mem_free(scr->data[y].u.du);
+			}
+		mem_free(scr->data);
+		free_list(struct compressed_chars, scr->compressed_chars);
+	}
 	if (scr->lines1) mem_free(scr->lines1);
 	if (scr->lines2) mem_free(scr->lines2);
 	if (scr->opt.framename) mem_free(scr->opt.framename);
@@ -169,7 +176,7 @@ static void clear_formatted(struct f_data *scr)
 void destroy_formatted(struct f_data *scr)
 {
 	if (scr->fd) {
-		internal("trying to free locked formatted data");
+		internal_error("trying to free locked formatted data");
 		return;
 	}
 	clear_formatted(scr);
@@ -220,7 +227,7 @@ int find_nearest_color(struct rgb *r, int l)
 	static int cache_init = 0;
 	int h;
 	if ((size_t)l > array_elements(palette_16_colors))
-		internal("invalid length %d", l);
+		internal_error("invalid length %d", l);
 	if (!cache_init) goto initialize;
 	back:
 	h = HASH_RGB(r, l);
@@ -279,7 +286,7 @@ static void xpand_lines(struct part *p, int y)
 		for (i = p->data->y; i < y; i++) {
 			p->data->data[i].l = 0;
 			p->data->data[i].allocated = 0;
-			p->data->data[i].d = DUMMY;
+			p->data->data[i].u.du = DUMMY;
 		}
 		p->data->y = y;
 	}
@@ -293,7 +300,7 @@ static void xpand_line(struct part *p, int y, int x)
 	y = safe_add(y, p->yp);
 #ifdef DEBUG
 	if (y >= p->data->y) {
-		internal("line does not exist");
+		internal_error("line does not exist");
 		return;
 	}
 #endif
@@ -304,11 +311,11 @@ static void xpand_line(struct part *p, int y, int x)
 			if (x >= 0x4000) ln->allocated = safe_add(x, x);
 			else ln->allocated = safe_add(x, 0x10) & ~0xf;
 			if ((unsigned)ln->allocated > MAXINT / sizeof(chr)) overalloc();
-			ln->d = mem_realloc(ln->d, ln->allocated*sizeof(chr));
+			ln->u.du = mem_realloc(ln->u.du, ln->allocated * sizeof(chr));
 		}
 		for (i = ln->l; i <= x; i++) {
-			ln->d[i].at = p->attribute;
-			ln->d[i].ch = ' ';
+			ln->u.du[i].at = p->attribute;
+			ln->u.du[i].ch = ' ';
 		}
 		ln->l = i;
 	}
@@ -329,7 +336,7 @@ static inline void xpand_spaces(struct part *p, int l)
 	if ((unsigned)l >= (unsigned)p->spl) r_xpand_spaces(p, l);
 }
 
-#define POS(x, y) (p->data->data[safe_add(p->yp, (y))].d[safe_add(p->xp, (x))])
+#define POS(x, y) (p->data->data[safe_add(p->yp, (y))].u.du[safe_add(p->xp, (x))])
 #define LEN(y) (p->data->data[safe_add(p->yp, (y))].l - p->xp < 0 ? 0 : p->data->data[p->yp + (y)].l - p->xp)
 #define SLEN(y, x) p->data->data[safe_add(p->yp, (y))].l = safe_add(p->xp, x)
 #define X(x) safe_add(p->xp, (x))
@@ -349,7 +356,7 @@ static inline void set_hchars(struct part *p, int x, int y, int xl, unsigned ch,
 {
 	chr *cc;
 	xpand_lines(p, y);
-	xpand_line(p, y, safe_add(x,xl)-1);
+	xpand_line(p, y, safe_add(x, xl) - 1);
 	cc = &POS(x, y);
 	for (; xl; xl--) {
 		cc->ch = ch;
@@ -525,7 +532,7 @@ static int split_line(struct part *p)
 	if (safe_add(i, par_format.rightmargin) > p->x) p->x = i + par_format.rightmargin;
 	if (p->data) {
 #ifdef DEBUG
-		if (POS(i, p->cy).ch != ' ') internal("bad split: %c", (unsigned char)POS(i, p->cy).ch);
+		if (POS(i, p->cy).ch != ' ') internal_error("bad split: %c", (unsigned char)POS(i, p->cy).ch);
 #endif
 		move_chars(p, safe_add(i,1), p->cy, par_format.leftmargin, safe_add(p->cy,1));
 		del_chars(p, i, p->cy);
@@ -541,14 +548,56 @@ static int split_line(struct part *p)
 	return 1 + (p->cx == -1);
 }
 
+static inline void compress_line(struct f_data *f, int y)
+{
+	int i, want_len;
+	struct line *ln = &f->data[y];
+	struct compressed_chars *cs;
+	for (i = 0; i < ln->l; i++) {
+		if (ln->u.du[i].ch + 0 >= 256)
+			return;
+		if (ln->u.du[i].at != ln->u.du[0].at)
+			return;
+	}
+	if (!list_empty(f->compressed_chars)) {
+		cs = list_struct(f->compressed_chars.next, struct compressed_chars);
+		if (ln->l > cs->total_len - cs->cpos)
+			goto out_of_space;
+		goto fill_it;
+	}
+out_of_space:
+	want_len = (int)(page_size - alloc_overhead - offsetof(struct compressed_chars, chars));
+	if (want_len < ln->l)
+		want_len = ln->l;
+	cs = mem_alloc_mayfail(offsetof(struct compressed_chars, chars) + want_len);
+	if (!cs)
+		return;
+	cs->total_len = want_len;
+	cs->cpos = 0;
+	add_to_list(f->compressed_chars, cs);
+fill_it:
+	for (i = 0; i < ln->l; i++) {
+		cs->chars[cs->cpos + i] = (unsigned char)ln->u.du[i].ch;
+	}
+	if (ln->l)
+		ln->allocated = -0x100 + ln->u.du[0].at;
+	else
+		ln->allocated = -0x100;
+	mem_free(ln->u.du);
+	ln->u.dc = &cs->chars[cs->cpos];
+	cs->cpos += ln->l;
+}
+
 static void align_line(struct part *p, int y)
 {
 	int na;
 	if (!p->data) return;
-	if (!LEN(y) || par_format.align == AL_LEFT || par_format.align == AL_NO || par_format.align == AL_NO_BREAKABLE || par_format.align == AL_BLOCK /* !!! fixme! */) return;
+	if (!LEN(y) || par_format.align == AL_LEFT || par_format.align == AL_NO || par_format.align == AL_NO_BREAKABLE || par_format.align == AL_BLOCK /* !!! fixme! */) goto compr;
 	na = rm(par_format) - LEN(y);
 	if (par_format.align == AL_CENTER) na /= 2;
 	if (na > 0) shift_chars(p, y, na);
+compr:
+	if (!table_level) compress_line(p->data, y);
 }
 
 struct link *new_link(struct f_data *f)
@@ -729,7 +778,7 @@ static void put_chars(void *p_, unsigned char *c, int l)
 		if (!p->data) goto x;
 		link = &p->data->links[p->data->nlinks - 1];
 		if (!p->data->nlinks) {
-			internal("no link");
+			internal_error("no link");
 			goto no_l;
 		}
 		goto set_link;
@@ -756,7 +805,7 @@ static void put_chars(void *p_, unsigned char *c, int l)
 				snzprint(s + 1, 62, p->link_num);
 				strcat(cast_char s, "]");
 			} else {
-				if (ff && (ff->type == FC_TEXT || ff->type == FC_PASSWORD || ff->type == FC_FILE || ff->type == FC_TEXTAREA)) {
+				if (ff && (ff->type == FC_TEXT || ff->type == FC_PASSWORD || ff->type == FC_FILE_UPLOAD || ff->type == FC_TEXTAREA)) {
 					strcpy(cast_char s, ">");
 				} else if (ff && (ff->type == FC_CHECKBOX || ff->type == FC_RADIO || ff->type == FC_SELECT)) {
 					strcpy(cast_char s, "");
@@ -787,7 +836,7 @@ static void put_chars(void *p_, unsigned char *c, int l)
 			link->where = stracpy(last_link);
 			link->target = stracpy(last_target);
 		} else {
-			link->type = last_form->type == FC_TEXT || last_form->type == FC_PASSWORD || last_form->type == FC_FILE ? L_FIELD : last_form->type == FC_TEXTAREA ? L_AREA : last_form->type == FC_CHECKBOX || last_form->type == FC_RADIO ? L_CHECKBOX : last_form->type == FC_SELECT ? L_SELECT : L_BUTTON;
+			link->type = last_form->type == FC_TEXT || last_form->type == FC_PASSWORD || last_form->type == FC_FILE_UPLOAD ? L_FIELD : last_form->type == FC_TEXTAREA ? L_AREA : last_form->type == FC_CHECKBOX || last_form->type == FC_RADIO ? L_CHECKBOX : last_form->type == FC_SELECT ? L_SELECT : L_BUTTON;
 			link->form = last_form;
 			link->target = stracpy(last_form->target);
 		}
@@ -819,6 +868,7 @@ static void put_chars(void *p_, unsigned char *c, int l)
 		fg = fg_color(fg, bg);
 		if (format_.attr & AT_ITALIC) fg = fg ^ 0x01;
 		if (format_.attr & AT_UNDERLINE) fg = (fg ^ 0x04) | 0x08;
+		if (format_.attr & AT_STRIKE) fg = (fg ^ 0x06);
 		if (format_.attr & AT_BOLD) fg = fg | 0x08;
 		fg = fg_color(fg, bg);
 		if (format_.attr & AT_GRAPHICS) bg = bg | 0x10;
@@ -842,7 +892,7 @@ static void line_break(void *p_)
 	if (!p->data) goto e;
 	xpand_lines(p, safe_add(p->cy, 1));
 	if (p->cx > par_format.leftmargin && LEN(p->cy) > p->cx - 1 && POS(p->cx-1, p->cy).ch == ' ') del_chars(p, p->cx-1, p->cy), p->cx--;
-	if (p->cx > 0) align_line(p, p->cy);
+	if (p->cx >= 0) align_line(p, p->cy);
 	if (p->data) foreachbackfrom(struct tag, t, lt, p->data->tags, last_tag_for_newline) {
 		t->x = X(0);
 		t->y = Y(p->cy + 1);
@@ -899,7 +949,7 @@ struct frameset_desc *create_frameset(struct f_data *fda, struct frameset_param
 	int i;
 	struct frameset_desc *fd;
 	if (!fp->x || !fp->y) {
-		internal("zero size of frameset");
+		internal_error("zero size of frameset");
 		return NULL;
 	}
 	if (fp->x && (unsigned)fp->x * (unsigned)fp->y / (unsigned)fp->x != (unsigned)fp->y) overalloc();
@@ -1011,9 +1061,15 @@ static void *html_special(void *p_, int c, ...)
 			va_end(l);
 			if (p->data) set_base(p->data, t);
 			break;
+		case SP_FORCE_BREAK:
+			if (p->cx > par_format.leftmargin) {
+				nobreak = 0;
+				line_break(p);
+			}
+			break;
 		default:
 			va_end(l);
-			internal("html_special: unknown code %d", c);
+			internal_error("html_special: unknown code %d", c);
 	}
 	return NULL;
 }
@@ -1048,7 +1104,7 @@ struct part *format_html_part(unsigned char *start, unsigned char *end, int alig
 		if (p) return p;
 	}
 	if (ys < 0) {
-		internal("format_html_part: ys == %d", ys);
+		internal_error("format_html_part: ys == %d", ys);
 		return NULL;
 	}
 	if (data) {
@@ -1113,7 +1169,7 @@ struct part *format_html_part(unsigned char *start, unsigned char *end, int alig
 	while (&html_top != e) {
 		kill_html_stack_item(&html_top);
 		if (!&html_top || (void *)&html_top == (void *)&html_stack) {
-			internal("html stack trashed");
+			internal_error("html stack trashed");
 			break;
 		}
 	}
@@ -1149,7 +1205,7 @@ static void push_base_format(unsigned char *url, struct document_options *opt, i
 {
 	struct html_element *e;
 	if (!list_empty(html_stack)) {
-		internal("something on html stack");
+		internal_error("something on html stack");
 		init_list(html_stack);
 	}
 	e = mem_calloc(sizeof(struct html_element));
@@ -1285,10 +1341,16 @@ void really_format_html(struct cache_entry *ce, unsigned char *start, unsigned c
 	if (!F) {
 		screen->x = 0;
 		for (i = screen->y - 1; i >= 0; i--) {
-			if (!screen->data[i].l) mem_free(screen->data[i].d), screen->y--;
-			else break;
+			if (!screen->data[i].l) {
+				if (screen->data[i].allocated >= 0)
+					mem_free(screen->data[i].u.du);
+				screen->y--;
+			} else {
+				break;
+			}
 		}
 		for (i = 0; i < screen->y; i++) if (screen->data[i].l > screen->x) screen->x = screen->data[i].l;
+		screen->data = mem_realloc(screen->data, screen->y * sizeof(struct line));
 	}
 	if (form.action) mem_free(form.action), form.action = NULL;
 	if (form.target) mem_free(form.target), form.target = NULL;
@@ -1300,7 +1362,7 @@ void really_format_html(struct cache_entry *ce, unsigned char *start, unsigned c
 	screen->bg = get_attribute(fg_col, bg_col);
 	kill_html_stack_item(&html_top);
 	if (!list_empty(html_stack)) {
-		internal("html stack not empty after operation");
+		internal_error("html stack not empty after operation");
 		init_list(html_stack);
 	}
 	sort_links(screen);
@@ -1310,10 +1372,6 @@ void really_format_html(struct cache_entry *ce, unsigned char *start, unsigned c
 
 int compare_opt(struct document_options *o1, struct document_options *o2)
 {
-#ifdef G
-	double kdo_si_hraje_nezlobi____a_nebo_to_je_PerM=o1->bfu_aspect-o2->bfu_aspect;
-#endif
-
 	if (o1->xw == o2->xw &&
 	    o1->yw == o2->yw &&
 	    o1->xp == o2->xp &&
@@ -1339,13 +1397,10 @@ int compare_opt(struct document_options *o1, struct document_options *o2)
 	    o1->display_images == o2->display_images &&
 	    o1->image_scale == o2->image_scale &&
 	    o1->porn_enable == o2->porn_enable &&
+	    o1->gamma_stamp == o2->gamma_stamp &&
 	    !memcmp(&o1->default_fg, &o2->default_fg, sizeof(struct rgb)) &&
 	    !memcmp(&o1->default_bg, &o2->default_bg, sizeof(struct rgb)) &&
 	    !memcmp(&o1->default_link, &o2->default_link, sizeof(struct rgb)) &&
-#ifdef G
-	    kdo_si_hraje_nezlobi____a_nebo_to_je_PerM<=0.000001 &&
-	    kdo_si_hraje_nezlobi____a_nebo_to_je_PerM>=-0.000001 &&
-#endif
 	    ((o1->framename && o2->framename && !casestrcmp(o1->framename, o2->framename)) || (!o1->framename && !o2->framename))) return 0;
 	return 1;
 }
@@ -1403,11 +1458,6 @@ static int sort_srch(struct f_data *f)
 	return 0;
 }
 
-static inline int is_spc(chr *cc)
-{
-	return cc->ch <= ' ' || cc->at & ATTR_FRAME;
-}
-
 static int n_chr, n_pos;
 static char_t srch_last_chr;
 static int srch_last_x, srch_last_y;
@@ -1461,6 +1511,14 @@ static int add_srch_chr(struct f_data *f, unsigned c, int x, int y, int nn)
 	return 0;
 }
 
+static inline int is_spc(struct f_data *f, int x, int y)
+{
+	unsigned ch;
+	unsigned char at;
+	get_char_attr(f, x, y, &ch, &at);
+	return ch <= ' ' || at & ATTR_FRAME;
+}
+
 static int get_srch(struct f_data *f)
 {
 	struct node *n;
@@ -1477,10 +1535,15 @@ do {						\
 		/*printf("%d %d - %d %d\n", n->x, n->y, xm, ym);
 		fflush(stdout);*/
 		for (y = n->y; y < ym && y < f->y; y++) {
+			struct line *ln = &f->data[y];
 			int ns = 1;
-			for (x = n->x; x < xm && x < f->data[y].l; x++) {
-				unsigned c = f->data[y].d[x].ch;
-				if (is_spc(&f->data[y].d[x])) c = ' ';
+			for (x = n->x; x < xm && x < ln->l; x++) {
+				unsigned c;
+				if (ln->allocated >= 0)
+					c = ln->u.du[x].ch;
+				else
+					c = ln->u.dc[x];
+				if (is_spc(f, x, y)) c = ' ';
 				if (c == ' ' && ns) continue;
 				c = charset_upcase(c, f->opt.cp);
 				if (ns) {
@@ -1492,7 +1555,7 @@ do {						\
 					add_srch(c, x, y, 1);
 				} else {
 					int xx;
-					for (xx = safe_add(x, 1); xx < xm && xx < f->data[y].l; xx++) if (!is_spc(&f->data[y].d[xx])) goto ja_uz_z_toho_programovani_asi_zcvoknu;
+					for (xx = safe_add(x, 1); xx < xm && xx < f->data[y].l; xx++) if (!is_spc(f, xx, y)) goto ja_uz_z_toho_programovani_asi_zcvoknu;
 					xx = x;
 					ja_uz_z_toho_programovani_asi_zcvoknu:
 				/* uz jsem zcvoknul, trpim poruchou osobnosti */
@@ -1522,7 +1585,7 @@ int get_search_data(struct f_data *f)
 		f->search_chr = NULL;
 		return -1;
 	}
-	if (get_srch(f)) internal("get_search_data: get_srch should not fail second time");
+	if (get_srch(f)) internal_error("get_search_data: get_srch should not fail second time");
 	while (n_chr && f->search_chr[n_chr - 1] == ' ') n_chr--;
 	f->nsearch_chr = n_chr;
 	f->nsearch_pos = n_pos;
