@@ -551,7 +551,7 @@ static void add_user_agent(unsigned char **hdr, int *l, unsigned char *url)
 {
 	add_to_str(hdr, l, cast_uchar "User-Agent: ");
 	if (SCRUB_HEADERS) {
-		add_to_str(hdr, l, cast_uchar "Mozilla/5.0 (Windows NT 10.0; rv:91.0) Gecko/20100101 Firefox/91.0\r\n");
+		add_to_str(hdr, l, cast_uchar "Mozilla/5.0 (Windows NT 10.0; rv:102.0) Gecko/20100101 Firefox/102.0\r\n");
 	} else if (!(*http_options.header.fake_useragent)) {
 
 		/*
@@ -661,7 +661,7 @@ static void add_accept(unsigned char **hdr, int *l, struct connection *c)
 	if (c->doh) {
 		add_to_str(hdr, l, cast_uchar "Accept: application/dns-message\r\n");
 	} else if (SCRUB_HEADERS) {
-		add_to_str(hdr, l, cast_uchar "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8\r\n");
+		add_to_str(hdr, l, cast_uchar "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8\r\n");
 	} else {
 		add_to_str(hdr, l, cast_uchar "Accept: */*\r\n");
 	}
