@@ -60,6 +60,16 @@ static void destroy_decoder (struct cached_image *cimg)
 		case IM_SVG:
 			svg_destroy_decoder(cimg);
 			break;
+#endif
+#ifdef HAVE_WEBP
+		case IM_WEBP:
+			webp_destroy_decoder(cimg);
+			break;
+#endif
+#ifdef HAVE_AVIF
+		case IM_AVIF:
+			avif_destroy_decoder(cimg);
+			break;
 #endif
 		}
 		mem_free(cimg->decoder);
@@ -83,7 +93,7 @@ static void img_destruct_image(struct g_object *object)
 	if (goi->list_entry.next) del_from_list(goi);
 	if (goi->goti.go.xw && goi->goti.go.yw) {
 		 /* At least one dimension is zero */
-		goi->cimg->refcount--;
+		deref_cached_image(goi->cimg);
 	}
 	mem_free(goi);
 }
@@ -773,6 +783,21 @@ static void type(struct cached_image *cimg, unsigned char *content_type, unsigne
 #ifdef HAVE_SVG
 	if (data[0] == '<' && data[1] == '?')
 		goto have_svg;
+#endif
+#ifdef HAVE_WEBP
+	if (data[0] == 'R' && data[1] == 'I' && data[2] == 'F' && data[3] == 'F')
+		goto have_webp;
+#endif
+#if 0
+	if (data[4] == 'f' && data[5] == 't' && data[6] == 'y' && data[7] == 'p' &&
+       (memcmp(data + 8, "avi", 3) &&
+        (data[11] == 'f' || data[11] == 's' || data[11] == 'o')) ||
+       (data[8] == 'm' &&
+        (data[9] == 'i' || data[9] == 's') &&
+        data[10] == 'f' && data[11] == '1') ||
+       memcmp(data + 8, "miaf", 4) ||
+       memcmp(data + 8, "MA1B", 4))
+		goto have_avif;
 #endif
 	if (data[0] == 0x89 && data[1] == 'P' && data[2] == 'N' && data[3] == 'G')
 		goto have_png;
@@ -819,6 +844,20 @@ static void type(struct cached_image *cimg, unsigned char *content_type, unsigne
 		svg_start(cimg);
 	} else
 #endif /* #ifdef HAVE_SVG */
+#ifdef HAVE_WEBP
+	if (dtest(cast_uchar "image/webp",content_type)) {
+		have_webp:
+		cimg->image_type=IM_WEBP;
+		webp_start(cimg);
+	} else
+#endif /* #ifdef HAVE_WEBP */
+#ifdef HAVE_AVIF
+	if (dtest(cast_uchar "image/avif",content_type)) {
+		/*have_avif:*/
+		cimg->image_type=IM_AVIF;
+	    avif_start(cimg);
+	} else
+#endif /* #ifdef HAVE_AVIF */
 	{
 		/* Error */
 		img_end(cimg);
@@ -916,6 +955,16 @@ static int img_process_download(struct g_object_image *goi, struct f_data_c *fda
 			svg_restart(cimg,data,length);
 			break;
 #endif /* #ifdef HAVE_SVG */
+#ifdef HAVE_WEBP
+		case IM_WEBP:
+			webp_restart(cimg,data,length);
+			break;
+#endif /* #ifdef HAVE_WEBP */
+#ifdef HAVE_AVIF
+		case IM_AVIF:
+			avif_restart(cimg,data,length);
+			break;
+#endif /* #ifdef HAVE_AVIF */
 #ifdef DEBUG
 		default:
 			fprintf(stderr,"cimg->image_type=%d\n",cimg->state);
@@ -942,6 +991,14 @@ img_process_download.\n");
 #ifdef HAVE_SVG
 				if (cimg->image_type==IM_SVG)
 					svg_finish(cimg);
+#endif
+#ifdef HAVE_WEBP
+				if (cimg->image_type==IM_WEBP)
+					webp_finish(cimg);
+#endif
+#ifdef HAVE_AVIF
+				if (cimg->image_type==IM_AVIF)
+					avif_finish(cimg);
 #endif
 			}
 			cimg->eof_hit=1;
@@ -1176,7 +1233,6 @@ static void find_or_make_cached_image(struct g_object_image *image, unsigned cha
 		image->goti.go.yw, image->xyw_meaning, scale, aspect))){
 		/* We have to make a new image cache entry */
 		cimg = mem_alloc(sizeof(*cimg));
-		cimg->refcount = 1;
 		cimg->background_color = image->background;
 #ifdef DEBUG
 		if (!url)
@@ -1291,7 +1347,7 @@ next_chunk:
 	if (retval && !(cimg->state & 4)) goto next_chunk;
 	image->goti.go.xw = (int)image->cimg->xww;
 	image->goti.go.yw = (int)image->cimg->yww;
-	if (cimg->state == 0 || cimg->state == 8 || (!image->af->rq->ce && image->af->unknown_image_size)) if (image->af->need_reparse != -1) image->af->need_reparse = 1;
+	if (cimg->state == 0 || cimg->state == 8 || image->af->unknown_image_size) if (image->af->need_reparse != -1) image->af->need_reparse = 1;
 	if (im->insert_flag) add_to_list(current_f_data->images, image);
 	else image->list_entry.prev = image->list_entry.next = NULL;
 	return image;
@@ -1309,7 +1365,7 @@ void change_image (struct g_object_image *goi, unsigned char *url, unsigned char
 	if (goi->orig_src) mem_free(goi->orig_src);
 	goi->orig_src = stracpy(src);
 	if (!(goi->goti.go.xw && goi->goti.go.yw)) return;
-	goi->cimg->refcount--;
+	deref_cached_image(goi->cimg);
 	goi->af = request_additional_file(fdata, url);
 	goi->af->need_reparse = -1;
 
@@ -1346,6 +1402,12 @@ int known_image_type(unsigned char *type)
 	if (!casestrcmp(type, cast_uchar "image/svg+xml")) return 1;
 	if (!casestrcmp(type, cast_uchar "image/svg")) return 1;
 #endif
+#ifdef HAVE_WEBP
+	if (!casestrcmp(type, cast_uchar "image/webp")) return 1;
+#endif
+#ifdef HAVE_AVIF
+	if (!casestrcmp(type, cast_uchar "image/avif")) return 1;
+#endif
 #endif
 	return 0;
 }
