@@ -127,6 +127,9 @@ void setcstate(struct connection *c, int state)
 {
 	struct status *stat;
 	struct list_head *lstat;
+	if (state < 0)
+		if (c->cache)
+			finish_cache_entry(c->cache);
 	if (c->state < 0 && state >= 0) c->prev_error = c->state;
 	if ((c->state = state) == S_TRANS) {
 		struct remaining_info *r = &c->prg;
