@@ -253,7 +253,7 @@ void smb_func(struct connection *c)
 			}
 			break;
 		default:
-			internal("unsupported smb client");
+			internal_error("unsupported smb client");
 		}
 		v[n++] = NULL;
 		EINTRLOOP(rs, execvp(cast_const_char v[0], (void *)v));
