@@ -817,9 +817,9 @@ static inline int getcompcode(int c)
 	return (c<<1 | (c&4)>>2) & 7;
 }
 
-unsigned char frame_dumb[49] =	"   ||||++||++++++--|-+||++--|-+----++++++++     ";
-static unsigned char frame_vt100[49] =	"aaaxuuukkuxkjjjkmvwtqnttmlvwtqnvvwwmmllnnjla    ";
-static unsigned char frame_koi[48] = {
+unsigned char frame_dumb[49] = "   ||||++||++++++--|-+||++--|-+----++++++++     ";
+static const unsigned char frame_vt100[49] = "aaaxuuukkuxkjjjkmvwtqnttmlvwtqnvvwwmmllnnjla    ";
+static const unsigned char frame_koi[48] = {
 	144,145,146,129,135,178,180,167,
 	166,181,161,168,174,173,172,131,
 	132,137,136,134,128,138,175,176,
@@ -827,7 +827,7 @@ static unsigned char frame_koi[48] = {
 	186,182,183,170,169,162,164,189,
 	188,133,130,141,140,142,143,139,
 };
-static unsigned char frame_freebsd[48] = {
+static const unsigned char frame_freebsd[48] = {
 	130,138,128,153,150,150,150,140,
 	140,150,153,140,139,139,139,140,
 	142,151,152,149,146,143,149,149,
@@ -835,7 +835,15 @@ static unsigned char frame_freebsd[48] = {
 	151,152,152,142,142,141,141,143,
 	143,139,141,128,128,128,128,128,
 };
-static unsigned char frame_restrict[48] = {
+static const unsigned short frame_utf8[48] = {
+	0x2591,0x2592,0x2593,0x2502,0x2524,0x2561,0x2562,0x2556,
+	0x2555,0x2563,0x2551,0x2557,0x255d,0x255c,0x255b,0x2510,
+	0x2514,0x2534,0x252c,0x251c,0x2500,0x253c,0x255e,0x255f,
+	0x255a,0x2554,0x2569,0x2566,0x2560,0x2550,0x256c,0x2567,
+	0x2568,0x2564,0x2565,0x2559,0x2558,0x2552,0x2553,0x256b,
+	0x256a,0x2518,0x250c,0x2588,0x2584,0x258c,0x2590,0x2580,
+};
+static const unsigned char frame_restrict[48] = {
 	  0,  0,  0,  0,  0,179,186,186,
 	205,  0,  0,  0,  0,186,205,  0,
 	  0,  0,  0,  0,  0,  0,179,186,
@@ -882,6 +890,9 @@ static inline char_t utf8_hack(char_t c)
 	char_t c = term->screen[p].ch;					\
 	unsigned char A = term->screen[p].at & 0x7f;			\
 	unsigned char frm = !!(term->screen[p].at & ATTR_FRAME);	\
+	if (s->restrict_852 && frm && c >= 176 && c < 224) {		\
+		if (frame_restrict[c - 176]) c = frame_restrict[c - 176];\
+	}								\
 	if (s->mode == TERM_LINUX) {					\
 		if (s->m11_hack) {					\
 			if (frm != mode) {				\
@@ -889,9 +900,6 @@ static inline char_t utf8_hack(char_t c)
 				else add_to_str(&a, &l, cast_uchar "\033[11m");\
 			}						\
 		}							\
-		if (s->restrict_852 && frm && c >= 176 && c < 224) {	\
-			if (frame_restrict[c - 176]) c = frame_restrict[c - 176];\
-		}							\
 	} else if (s->mode == TERM_VT100) {				\
 		if (frm != mode) {					\
 			if (!(mode = frm)) add_to_str(&a, &l, cast_uchar "\017");\
@@ -900,7 +908,8 @@ static inline char_t utf8_hack(char_t c)
 		if (frm && c >= 176 && c < 224) c = frame_vt100[c - 176];\
 	} else if (s->mode == TERM_KOI8 && frm && c >= 176 && c < 224) { c = frame_koi[c - 176];\
 	} else if (s->mode == TERM_FREEBSD && frm && c >= 176 && c < 224) { c = frame_freebsd[c - 176];\
-	} else if (s->mode == TERM_DUMB && frm && c >= 176 && c < 224) c = frame_dumb[c - 176];\
+	} else if (s->mode == TERM_UTF8 && frm && c >= 176 && c < 224 && term_charset(term) == utf8_table) { c = frame_utf8[c - 176]; frm = 0;\
+	} else if (frm && c >= 176 && c < 224) c = frame_dumb[c - 176];\
 	if (!(A & 0100) && (A >> 3) == (A & 7)) A = (A & 070) | 7 * !(A & 020);\
 	if (A != attrib) {						\
 		attrib = A;						\
@@ -1325,14 +1334,14 @@ void exec_on_terminal(struct terminal *term, unsigned char *path, unsigned char
 	int rs;
 	if (path && !*path) return;
 	if (!path) path = cast_uchar "";
-#ifdef NO_FG_EXEC
-	fg = 0;
-#endif
 #ifdef HAVE_EXE_ON_BACKGROUND
 	if (*path) {
-		rs = exe_on_background(path, delet);
+		rs = exe_on_background(path, delet, fg);
 		if (!rs) return;
 	}
+#endif
+#ifdef NO_FG_EXEC
+	fg = 0;
 #endif
 	if (term->master) {
 		if (!*path) {
