@@ -750,6 +750,7 @@ void create_initial_extensions(void)
 	ext.ext = cast_uchar "xpm", ext.ct=cast_uchar "image/x-xpixmap", update_ext(&ext);
 	ext.ext = cast_uchar "xls", ext.ct=cast_uchar "application/excel", update_ext(&ext);
 	ext.ext = cast_uchar "xbm", ext.ct=cast_uchar "image/x-xbitmap", update_ext(&ext);
+	ext.ext = cast_uchar "webp", ext.ct=cast_uchar "image/webp", update_ext(&ext);
 	ext.ext = cast_uchar "wav", ext.ct=cast_uchar "audio/x-wav", update_ext(&ext);
 	ext.ext = cast_uchar "tiff,tif", ext.ct=cast_uchar "image/tiff", update_ext(&ext);
 	ext.ext = cast_uchar "tga", ext.ct=cast_uchar "image/targa", update_ext(&ext);
@@ -784,6 +785,7 @@ void create_initial_extensions(void)
 	ext.ext = cast_uchar "dvi", ext.ct=cast_uchar "application/x-dvi", update_ext(&ext);
 	ext.ext = cast_uchar "dl", ext.ct=cast_uchar "video/dl", update_ext(&ext);
 	ext.ext = cast_uchar "deb", ext.ct=cast_uchar "application/x-debian-package", update_ext(&ext);
+	ext.ext = cast_uchar "avif", ext.ct=cast_uchar "image/avif", update_ext(&ext);
 	ext.ext = cast_uchar "avi", ext.ct=cast_uchar "video/x-msvideo", update_ext(&ext);
 	ext.ext = cast_uchar "au,snd", ext.ct=cast_uchar "audio/basic", update_ext(&ext);
 	ext.ext = cast_uchar "aif,aiff,aifc", ext.ct=cast_uchar "audio/x-aiff", update_ext(&ext);
@@ -797,6 +799,7 @@ struct list_head telnet_prog = { &telnet_prog, &telnet_prog };
 struct list_head tn3270_prog = { &tn3270_prog, &tn3270_prog };
 struct list_head mms_prog = { &mms_prog, &mms_prog };
 struct list_head magnet_prog = { &magnet_prog, &magnet_prog };
+struct list_head gopher_prog = { &gopher_prog, &gopher_prog };
 
 
 static int is_in_list(unsigned char *list, unsigned char *str, int l)
@@ -900,6 +903,9 @@ unsigned char *get_content_type_by_extension(unsigned char *url)
 	if ((extl == 3 && !casecmp(ext, cast_uchar "xbm", 3))) return stracpy(cast_uchar "image/x-xbitmap");
 	if ((extl == 3 && !casecmp(ext, cast_uchar "tif", 3)) ||
 	    (extl == 4 && !casecmp(ext, cast_uchar "tiff", 4))) return stracpy(cast_uchar "image/tiff");
+	if ((extl == 3 && !casecmp(ext, cast_uchar "svg", 3))) return stracpy(cast_uchar "image/svg+xml");
+	if ((extl == 4 && !casecmp(ext, cast_uchar "webp", 4))) return stracpy(cast_uchar "image/webp");
+	if ((extl == 4 && !casecmp(ext, cast_uchar "avif", 4))) return stracpy(cast_uchar "image/avif");
 	exxt = init_str(); el = 0;
 	add_to_str(&exxt, &el, cast_uchar "application/x-");
 	add_bytes_to_str(&exxt, &el, ext, extl);
@@ -960,6 +966,10 @@ static unsigned char *get_extension_by_content_type(unsigned char *ct)
 	if (!casestrcmp(ct, cast_uchar "image/svg") ||
 	    !casestrcmp(ct, cast_uchar "image/svg+xml"))
 		return stracpy(cast_uchar "svg");
+	if (!casestrcmp(ct, cast_uchar "image/webp"))
+		return stracpy(cast_uchar "webp");
+	if (!casestrcmp(ct, cast_uchar "image/avif"))
+		return stracpy(cast_uchar "avif");
 	if (!cmpbeg(ct, cast_uchar "application/x-")) {
 		x = ct + strlen("application/x-");
 		if (casestrcmp(x, cast_uchar "z") &&
@@ -1317,8 +1327,8 @@ void free_types(void)
 	free_prog_list(&tn3270_prog);
 	free_prog_list(&mms_prog);
 	free_prog_list(&magnet_prog);
+	free_prog_list(&gopher_prog);
 
 	free_history(ext_search_history);
 	free_history(assoc_search_history);
 }
-
